#
# (c) 2022 Giorgio Gonnella, University of Goettingen, Germany
#
from multiplug.importer import importer
from multiplug.bash_mod import bash
from multiplug.nim_mod import nim
from multiplug.py_mod import py
from multiplug.rust_mod import rust
